#include <string.h>
#include <sys/types.h>
#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>
#include "strbuf.h"

string_buffer_t create_string_buffer(long long sz)
{
	long long rsz;
	string_buffer_t sb;
	sb = (string_buffer_t) calloc(sizeof(strbuf), 1);   
	rsz = LBX_STR_REALLOC_EX;
	if(sz > rsz) {
		rsz = sz;
	}   sb->size = rsz;
	sb->len = 0;
	sb->str = (char*) calloc(sizeof(char), rsz);
	return sb;
}

int delete_string_buffer(string_buffer_t sb)
{
	if(sb) {
		/* ok */
		free(sb->str);
		free(sb);
	}
	return 0;
}

long long clear_string_buffer(string_buffer_t sb)
{
	memset(sb->str, 0, sb->size);
	sb->len = 0;
	return sb->size;
}

long long append_buffer(string_buffer_t sb, const char* str)
{
	long long len = strlen(str);
	return append_buffer_n(sb, len, str);
}

long long append_buffer_n(string_buffer_t sb, long long size, const char* str)
{
	long long nsz, len = size;
	if((sb->len + len +1) > sb->size) {
		void* p;
		nsz = sb->size + len + LBX_STR_REALLOC_EX;
		p = realloc((void*) sb->str, nsz);
		memset((p + sb->len), 0, (nsz - sb->len));
		sb->size = nsz;
	}
	strncpy((sb->str + sb->len), str, len);
	sb->len = sb->len + len;
	return sb->size;
}

long long append_buffer_fmt(string_buffer_t sb, long long size, const char *fmt, ...)
{
	/* variable-argument front-end to append_buffer */
	va_list ap;
	char* str = calloc(sizeof(char), (size + 1));
	va_start(ap, fmt);
	vsnprintf(str, size, fmt, ap);
	append_buffer_n(sb, strlen(str), str); 
	va_end(ap);	
	free(str);
	return 0;
}

