#include "vacation.h"
#include "fv_milter_queue_client.h"
#include <unistd.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include <syslog.h>

extern int debug;

inline vacation_msg_t create_vacation_msg() {
	vacation_msg_t vm;
	vm = (vacation_msg_t) calloc(sizeof(vacation_msg), 1);	
	vm->msg = create_string_buffer(0);
	return vm;
}

inline int delete_vacation_msg(vacation_msg_t vm)
{
	delete_string_buffer(vm->msg);
	free(vm);
	return 0;
}

inline int append_to_vacation_msg(vacation_msg_t vm, const char* str)
{
	append_buffer(vm->msg, str);
	return 0;
}

inline int append_to_vacation_msg_fmt(vacation_msg_t vm, long long size, const char* fmt, ...)
{	
	/* variable-argument front-end to append_buffer */
	va_list ap;
	char* str = calloc(sizeof(char), (size + 1));
	va_start(ap, fmt);
	vsnprintf(str, size, fmt, ap);
	append_buffer_n(vm->msg, strlen(str), str); 
	va_end(ap);	
	free(str);
	return 0;
}

inline vacation_msg_list_t create_vacation_msg_list()
{
	vacation_msg_list_t vml;
	vml = (vacation_msg_list_t) calloc(sizeof(vacation_msg_list), 1);
	return vml;
}


inline int append_vacation_msg(vacation_msg_list_t vml, vacation_msg_t vm)
{
	vacation_msg_list* new_elem;
	vacation_msg_list* last;
	vacation_msg_list* iter = vml;
	while(iter) {
		last = iter;
		iter=iter->next;
	}
	if(!last->vmsg) {
		last->vmsg = vm;
	} else {
		new_elem = create_vacation_msg_list();
		new_elem->vmsg = vm;
		last->next = new_elem;
	}
	return 0;
}

inline long vacation_list_count(vacation_msg_list_t vml) {
	long ctr = 0;
	vacation_msg_list* iter = vml;
	while(iter) {
		++ctr;
		iter=iter->next;
	}
	return ctr;
}

inline int delete_vacation_msg_list(vacation_msg_list_t vml)
{
	long ix;
	long l = vacation_list_count(vml);
	vacation_msg_list** ppvml = (vacation_msg_list**) malloc(l * sizeof(vacation_msg_list*));
	vacation_msg_list* iter = vml;
	ix = 0;
	while(iter) {
		ppvml[ix] = iter;
		iter = iter->next;
		++ix;
	}
	for(ix = 0; ix < l; ++ix) {
		delete_vacation_msg(ppvml[ix]->vmsg);
		ppvml[ix]->vmsg = 0;
		free(ppvml[ix]);
	}
	free(ppvml);
	return 0;
}

int send_vacation_messages(const char *mail_cmd, vacation_msg_list_t vml, int debug) {
	/* traverse the list, sending the msgs therein */
	
	int status = 0;
		
	vacation_msg_list* iter = vml;
	while(iter) {
		sendmail_queue_message(iter->vmsg->msg->str, debug);
		iter=iter->next;
	}

	return status;
}

